#include <Inventor/Qt/SoQt.h>
#include <Inventor/Qt/viewers/SoQtExaminerViewer.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/events/SoMouseButtonEvent.h>
#include <Inventor/actions/SoRayPickAction.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/SoPickedPoint.h>



#include <iostream>


void mouse(void* userData, SoEventCallback* evt)
{
     SoQtExaminerViewer* viewer=(SoQtExaminerViewer*)userData;

     if ( SO_MOUSE_PRESS_EVENT(evt->getEvent(), ANY))
     {
	  // Action permettant de savoir quels objets du 
	  // graphe de scne ont t touchs par un rayon
	  // partant de la camra et passant par le point dsign 
	  //  la souris
	  SoRayPickAction rp( viewer->getViewportRegion() );
	  rp.setPoint(evt->getEvent()->getPosition());
	  rp.apply(viewer->getSceneManager()->getSceneGraph());
	  SoPickedPoint* pp = rp.getPickedPoint();

	  if (pp == NULL)
	  {
	       evt->setHandled();
	       return;
	  }

	  SoNode* node = pp->getPath()->getNodeFromTail(1);
	  if (node != NULL)
	  {
	       if (node->isOfType(SoSeparator::getClassTypeId()))
	       {
		    SoSeparator* sep = (SoSeparator*)node;
		    SoMaterial* mat = new SoMaterial;
		    mat->diffuseColor.setValue(1.0,0.0,0.0);
		    sep->insertChild(mat,0);
	       }
	  }


	  evt->setHandled();
	  return;
     }
}



int main(int argc, char** argv)
{
     QWidget* myWidget= SoQt::init(argc,argv,argv[0]);

     // Initialisation de la scne
     SoSeparator* root = new SoSeparator;
     root->ref(); // Augmenter le compteur de rfrences


     // Couleur gnrale
     SoMaterial*  mat = new SoMaterial ;
     mat->diffuseColor.setValue(0.0, 1.0, 1.0);

     // Routine d'interception des vnements
     SoEventCallback* eventCB = new SoEventCallback;
     root->addChild(eventCB);

     // Ajout d'une couleur
     root->addChild(mat);

     SoRotationXYZ* r0 = new SoRotationXYZ;
     r0->axis.setValue("X");
     r0->angle.setValue(0.30);
     root->addChild(r0);
     SoRotationXYZ* r1 = new SoRotationXYZ;
     r1->axis.setValue("Z");
     r1->angle.setValue(0.15);
     root->addChild(r1);


     SoSeparator* sep0 = new SoSeparator;
     SoTranslation* tr0 = new SoTranslation;
     tr0->translation.setValue(2.0, 2.0, 0.0);
     sep0->addChild(tr0);
     sep0->addChild(new SoCube);
     root->addChild(sep0);

     SoSeparator* sep1 = new SoSeparator;
     SoTranslation* tr1 = new SoTranslation;
     tr1->translation.setValue(2.0, -2.0, 0.0);
     sep1->addChild(tr1);
     sep1->addChild(new SoCube);
     root->addChild(sep1);

     SoSeparator* sep2 = new SoSeparator;
     SoTranslation* tr2 = new SoTranslation;
     tr2->translation.setValue(-2.0, -2.0, 0.0);
     sep2->addChild(tr2);
     sep2->addChild(new SoCube);
     root->addChild(sep2);

     SoSeparator* sep3 = new SoSeparator;
     SoTranslation* tr3 = new SoTranslation;
     tr3->translation.setValue(-2.0, 2.0, 0.0);
     sep3->addChild(tr3);
     sep3->addChild(new SoCube);
     root->addChild(sep3);

     
     SoQtExaminerViewer *myRenderArea = new SoQtExaminerViewer(myWidget);
     myRenderArea->setSceneGraph(root);
     myRenderArea->setViewing(false);
     myRenderArea->show();

     eventCB->addEventCallback( SoMouseButtonEvent::getClassTypeId(), 
     			mouse, myRenderArea);


     SoQt::show(myWidget);
     SoQt::mainLoop();

     delete myRenderArea;
     return 0;
}
