#include <Inventor/Qt/SoQt.h>
#include <Inventor/Qt/SoQtRenderArea.h>
  
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoRotor.h>
#include <Inventor/nodes/SoArray.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoFont.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/nodes/SoTranslation.h>


#include <Inventor/sensors/SoFieldSensor.h>

// Fonction de callback attach au capteur
// L'objectif est d'afficher l'angle de rotation
void sensorCallback(void* data, SoSensor* s)
{
     SoText2* txt = (SoText2*)data;

     SoFieldSensor* fs = (SoFieldSensor*)s;

     SoField* field = fs->getAttachedField();

     SbVec3f axis;
     float angle;

     ((SoSFRotation*)field)->getValue(axis,angle);

     SbString str;
     
     str.sprintf("angle : %f rad",angle);

     txt->string.setValue(str);
}

  
// Set up a simple scenegraph, just for demonstration purposes.
static SoSeparator * get_scene_graph()
{
     SoSeparator * root = new SoSeparator;
  
     SoGroup * group = new SoGroup;
  
     SoSeparator* sep = new SoSeparator;

     SoTranslation* tr = new SoTranslation;
     tr->translation.setValue(-4.0, 4.0, 0.0);
     sep->addChild(tr);

     SoText2* txt = new SoText2;
     txt->string.setValue("test");
     sep->addChild(txt);

     // Cration du capteur le paramtre data de la fonction
     // sensorCallback est un pointeur sur txt.
     SoFieldSensor* sensor= new SoFieldSensor(sensorCallback,txt);

     root->addChild(sep);

     SoRotor * rotor = new SoRotor;
     rotor->rotation = SbRotation(SbVec3f(0.2, 0.5, 0.9), M_PI/4.0);
     // Le capteur est attach sur le champ rotation du Rotor
     sensor->attach(rotor->getField(SbName("rotation") )); 

     group->addChild(rotor);
  
     SoCube * cube = new SoCube;
     group->addChild(cube);
  
     SoArray * array = new SoArray;
     array->origin = SoArray::CENTER;
     array->addChild(group);
     array->numElements1 = 2;
     array->numElements2 = 2;
     array->separation1 = SbVec3f(4, 0, 0);
     array->separation2 = SbVec3f(0, 4, 0);
  
     root->addChild(array);
     return root;
}
  
int
main(int argc, char ** argv)
{
     QWidget * window = SoQt::init(argc,argv, argv[0]);
  
     SoSeparator * root = new SoSeparator;
     root->ref();
  
     SoPerspectiveCamera * camera;
     root->addChild(camera = new SoPerspectiveCamera);
  
     root->addChild(new SoDirectionalLight);
  
     SoSeparator * userroot = get_scene_graph();
     root->addChild(userroot);
  
     SoQtRenderArea * renderarea = new SoQtRenderArea(window);
     camera->viewAll(userroot, renderarea->getViewportRegion());
     renderarea->setSceneGraph(root);
     renderarea->setBackgroundColor(SbColor(0.0f, 0.2f, 0.3f));
     if (argc > 1) {
	  renderarea->setTitle(argv[1]);
	  renderarea->setIconTitle(argv[1]);
     }
     renderarea->show();
  
     SoQt::show(window);
     SoQt::mainLoop();
  
     delete renderarea;
     root->unref();
  
     return 0;
}
