#include <stdlib.h>
#include <math.h>

#include <Inventor/Qt/SoQt.h>
#include <Inventor/Qt/viewers/SoQtExaminerViewer.h>
#include <Inventor/events/SoMouseButtonEvent.h>
#include <Inventor/actions/SoActions.h>
#include <Inventor/SoPickedPoint.h>
#include <Inventor/nodes/SoNodes.h>

#include <Inventor/manips/SoTrackballManip.h>
#include <Inventor/manips/SoCenterballManip.h>



//#include <qmessagebox.h>

SoSearchAction* sa;
SoCenterballManip* trackball;
SoEventCallback* eventCB;
SoPath* manipPath;
bool manipOn = false;







void mouse(void* userData, SoEventCallback* evt)
{
     SoQtExaminerViewer* viewer=(SoQtExaminerViewer*)userData;

     if ( SO_MOUSE_PRESS_EVENT(evt->getEvent(), ANY))
     {
	  // Action permettant de savoir quels objets du 
	  // graphe de scne ont t touchs par un rayon
	  // partant de la camra et passant par le point dsign 
	  //  la souris
	  bool right = SO_MOUSE_PRESS_EVENT(evt->getEvent(),BUTTON1);
	  SoRayPickAction rp( viewer->getViewportRegion() );
	  rp.setPoint(evt->getEvent()->getPosition());
	  rp.apply(viewer->getSceneManager()->getSceneGraph());
	  SoPickedPoint* pp = rp.getPickedPoint();

	  if (pp == NULL)
	  {
	       if (!manipOn)
		    evt->setHandled();
	       else
	       {
		    SoTransform* tr = new SoTransform;
		    trackball->replaceManip(manipPath,tr);
		    tr->scaleFactor.setValue(SbVec3f(1.0,1.0,1.0));
		    trackball->unref();
		    manipPath->unref();
		    manipOn = false;
	       }
	       return;
	  }

	  SoNode* node = pp->getPath()->getNodeFromTail(1);
	  if (node != NULL && !manipOn)
	  {
	       if (node->isOfType(SoSeparator::getClassTypeId()))
	       {
		    sa->reset();
		    sa->setType(SoMaterial::getClassTypeId());
		    sa->setInterest(SoSearchAction::FIRST);
		    sa->apply(node);
			 
		    SoMaterial* mat = (SoMaterial*)(sa->getPath()->getTail());
		    mat->diffuseColor.setValue(0.0,right?1.0:0.0,right?0.0:1.0);

		    sa->reset();
		    sa->setType(SoTransform::getClassTypeId());
		    sa->setInterest(SoSearchAction::FIRST);
		    sa->apply(node);
		    SoTransform* tr = (SoTransform*)(sa->getPath()->getTail());
		    tr->scaleFactor.setValue(SbVec3f(1.0,1.0,1.0));

		    sa->reset();
		    sa->setNode(tr);
		    sa->setInterest(SoSearchAction::FIRST);
		    sa->apply(viewer->getSceneManager()->getSceneGraph());
		    manipPath = sa->getPath();
		    manipPath->ref();

		    trackball = new SoCenterballManip ;
		    trackball->ref();


		    trackball->replaceNode(sa->getPath());
		    manipOn = true;
		    
		    evt->setHandled();
		    viewer->viewAll();
		    viewer->render();
	       }
	  }

	  if (!manipOn)
	       evt->setHandled();
	  return;
     }
}





SoNode* buildSceneGraph(SoQtExaminerViewer* myRenderArea)
{
     
     SoSeparator* root = new SoSeparator;
     root->ref(); // Augmenter le compteur de rfrences

     // Routine d'interception des vnements
     eventCB = new SoEventCallback;
     root->addChild(eventCB);

     eventCB->addEventCallback( SoMouseButtonEvent::getClassTypeId(), 
				mouse, myRenderArea);



     SoMaterial* vert = new SoMaterial;
     vert->diffuseColor.setValue(0.0, 1.0, 0.0);
     root->addChild(vert);
     SoSphere* sphere = new SoSphere;
     sphere->radius.setValue(0.2);
     root->addChild(sphere);

     SoTranslation* translate = new SoTranslation ;
     translate->translation.setValue(0,1.0,0);
	
     SoSphere* joint = new SoSphere;
     joint->radius.setValue(0.4);
     SoCylinder* cylinder = new SoCylinder;
     cylinder->radius.setValue(0.4);

     SoSeparator* ref =root;

     for (int i=0; i < 10; i++)
     {
	  SoSeparator* sep = new SoSeparator ;
	  ref->addChild(sep);
	  ref=sep;

	  SoMaterial* mat = new SoMaterial;
	  mat->diffuseColor.setValue(1.0,0.5,0.5);
	  sep->addChild(mat);
	  SoTransform* tr = new SoTransform ;
	  sep->addChild(tr);
	  sep->addChild(translate);
	  sep->addChild(cylinder);
	  sep->addChild(translate);
	  sep->addChild(joint);

     }

     return root;
}





int main(int argc, char** argv)
{
     QWidget* myWidget= SoQt::init(argc,argv,argv[0]);

     sa =new SoSearchAction();

     SoQtExaminerViewer *myRenderArea = new SoQtExaminerViewer(myWidget);

     myRenderArea->setSceneGraph(buildSceneGraph(myRenderArea));
     myRenderArea->show();

     myRenderArea->viewAll();
     myRenderArea->setViewing(false);


     SoQt::show(myWidget);

     SoQt::mainLoop();
     return 0;
}
