###############################################################################
# QT Project File for SoQt Inventor Files loader
###############################################################################

# -----------------------------------------------------------------------------
# QT Project Configuration
# -----------------------------------------------------------------------------
# Target name is based on the .pro filename
PRO_NAME = $$basename(_PRO_FILE_)
TARGET = $$replace(PRO_NAME, ".pro", "")
message(Generating Makefile for $${TARGET} application)

TEMPLATE = app
QT += core gui
CONFIG += link_pkgconfig
PKGCONFIG += SoQt

# Obsolete (use pkg-config instead)
# INCLUDEPATH+= $$system(soqt-config --includedir)
# LIBS+= $$system(soqt-config --ldflags --libs)

# name of classes (.cpp/.h) without extension (can be empty)
MODULES = ivloader
for(f, MODULES){
	HEADERS += $${f}.h
	SOURCES += $${f}.cpp
	ALLSOURCES += $${f}.h $${f}.cpp
}

# name of main programs (.cpp) without extension (cannot be empty)
MAINS = main
for(f, MAINS){
	SOURCES += $${f}.cpp
	ALLSOURCES += $${f}.cpp
}

OTHERFILES = ivdemo/

# special flag for yaka
QMAKE_LFLAGS_RPATH=""

# -----------------------------------------------------------------------------
# Extra tools
# -----------------------------------------------------------------------------

# Listing tools
A2PS = a2ps
PS2PDF = ps2pdf -sPAPERSIZE=a4
# Documentation tool
DOCTOOL = doxygen
# Year-month-day Date
unix {
	DATE = $$system(date +%Y-%m-%d)
}
win32 {
	DATE = $$system(for /F \"usebackq tokens=1,2,3 delims=/ \" %a in \
	(`date /t`) do @echo %c-%b-%a)
}

# Archive format
unix {
	ARCHIVER = tar
	ARCHOPT = zcvfH
	ARCHEXT = tgz
}
win32 {
	ARCHIVER = zip
	ARCHOPT =
	ARCHEXT = zip
}

# -----------------------------------------------------------------------------
# Extra targets
# -----------------------------------------------------------------------------

# documentation ---------------------------------------------------------------
doc.target = doc
doc.commands = $$DOCTOOL

# linking .h to .hpp for correct printing highlight needed by a2ps ------------
for(f, HEADERS){
	LINK_ACTION+="ln -fs $${f} $${f}pp; "
	CPPHEADERS+="$${f}pp "
}
links.target = links
links.commands = $$LINK_ACTION
links.depends = $$HEADERS

# unlinking .hpp --------------------------------------------------------------
for(f, HEADERS){UNLINK_ACTION+="rm -f $${f}pp; "}
unlinks.target = unlinks
unlinks.commands = $$UNLINK_ACTION

# Postscript listing generation with A2PS -------------------------------------
for(f, MODULES){
	PRINTSOURCES+= $${f}.hpp $${f}.cpp
}
for(f, MAINS){
	PRINTSOURCES+= $${f}.cpp
}
ps.target = ps
ps.commands = $$A2PS -2 --file-align=fill --line-numbers=1 --font-size=10 \
--chars-per-line=100 --tabsize=4 --pretty-print --highlight-level=heavy \
--prologue="gray" -o $${TARGET}.ps $${PRINTSOURCES}
ps.depends = links

# PDF listing generation from Postscript --------------------------------------
pdf.target = pdf
pdf.commands = $$PS2PDF $${TARGET}.ps $${TARGET}.pdf
pdf.depends = ps

# Timestamped archive generation ----------------------------------------------
archive.target = archive
archive.commands = $$ARCHIVER $$ARCHOPT $${TARGET}-$${DATE}.$${ARCHEXT} \
$${ALLSOURCES} $${TARGET}.pro $${TARGET}.pdf $${OTHERFILES}
archive.depends = pdf

# Cleaning doc, listing and archives ------------------------------------------
myclean.target = myclean
myclean.commands = rm -rf *~ $${TARGET}.ps $${TARGET}.pdf \
$${TARGET}*.$${ARCHEXT} doc
myclean.depends = unlinks

# Sources edition with default editor -----------------------------------------
edit.target = edit
edit.commands = $(EDITOR) $${ALLSOURCES} $${TARGET}.pro

# Adding extra clean to regular clean -----------------------------------------
clean.depends = myclean

# Extra targets to be added in the makefile -----------------------------------
QMAKE_EXTRA_TARGETS += doc ps pdf archive myclean links unlinks edit clean
# adding clean as extra targets when there is already a clean target generated
# by QT just adds a second clean target which is also executed when invoking
# make clean

# -----------------------------------------------------------------------------
# QT Information (uncomment for more informations)
# -----------------------------------------------------------------------------
# message(Qt version: $$[QT_VERSION])
# message(Qt is installed in $$[QT_INSTALL_PREFIX])
# message(Qt resources can be found in the following locations:)
# message(Documentation: $$[QT_INSTALL_DOCS])
# message(Header files: $$[QT_INSTALL_HEADERS])
# message(Libraries: $$[QT_INSTALL_LIBS])
# message(Binary files (executables): $$[QT_INSTALL_BINS])
# message(Plugins: $$[QT_INSTALL_PLUGINS])
# message(Data files: $$[QT_INSTALL_DATA])
# message(Translation files: $$[QT_INSTALL_TRANSLATIONS])
# message(Settings: $$[QT_INSTALL_SETTINGS])
# message(Examples: $$[QT_INSTALL_EXAMPLES])
# message(Demonstrations: $$[QT_INSTALL_DEMOS])
